<?php

namespace Drupal\commerce_payme;

use Drupal\commerce_order\Entity\Order;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Component\Datetime\Time;

/**
 * Handle payment return class.
 */
class HandlePaymentReturn {

  /**
   * Entity type manager.
   */
  protected $entityTypeManager;

  /**
   * Time service.
   */
  protected $time;

  /**
   * Construct.
   */
  public function __construct(EntityTypeManager $entity_type_manager, Time $time) {
    $this->entityTypeManager = $entity_type_manager;
    $this->time = $time;
  }

  /**
   * Handle payment return.
   *
   * @return int
   *   Return code:
   *   0: All good
   *   1: Decline exception.
   *   2: Authentication exception.
   *   3: Invalid response exception.
   */
  public function handle($order_id, $response) {
    $order = Order::load($order_id);
    $gateway = $order->payment_gateway->entity;
    $validate_response = $this->validateResponse($response, $gateway->getplugin());
    if ($validate_response === 0) {
      $amount = $order->getTotalPrice();
      $amount_number = number_format(round($amount->getNumber(), 2), 2, '', '');
        $payment_storage = $this->entityTypeManager->getStorage('commerce_payment');
        $payment = $payment_storage->create([
          'state' => 'authorization',
          'amount' => $amount,
          'payment_gateway' => $gateway,
          'order_id' => $order->id(),
          'test' => FALSE,
          'remote_id' => $response['authorizationCode'],
          'authorized' => $this->time->getRequestTime(),
        ]);
        $payment->save();
        return 0;
    }
    return $validate_response;
  }

  /**
   * Validate response array formatted from BNCR.
   *
   * @param array $response
   *   The response array.
   * @param mixed $payment_gateway
   *   The payment gateway.
   *
   * @return bool
   *   Whether response is valid or not.
   */
  protected function validateResponse(array $response, $payment_gateway) {
    if (isset($response['authorizationResult'])) {
      if ($response['authorizationResult'] === '01') {
        return 1;
      }
      elseif ($response['authorizationResult'] === '05') {
        return 2;
      }
    }
    else {
      return 3;
    }

    $hash_elements = [];
    $hash_elements[] = $payment_gateway->getAcquirerId();
    $hash_elements[] = $payment_gateway->getCommerceId();
    $hash_elements[] = $response['purchaseOperationNumber'];
    $hash_elements[] = $response['purchaseAmount'];
    $hash_elements[] = $payment_gateway->getCurrencyCode();
    $hash_elements[] = $response['authorizationResult'];
    $hash_elements[] = $payment_gateway->getKey();

    $hash = $payment_gateway->getHash($hash_elements);
    if (isset($response['purchaseVerification']) && $hash !== $response['purchaseVerification']) {
      return 3;
    }

    return 0;
  }
}