<?php

namespace Drupal\commerce_payme\Plugin\Commerce\PaymentGateway;

use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_payment\CreditCard;
use Drupal\commerce_payment\Entity\PaymentInterface;
use Drupal\commerce_payment\Entity\PaymentMethodInterface;
use Drupal\commerce_payment\PaymentMethodTypeManager;
use Drupal\commerce_payment\PaymentTypeManager;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayBase;
use Drupal\commerce_payment\Exception\PaymentGatewayException;
use Drupal\commerce_payment\Exception\InvalidResponseException;
use Drupal\commerce_payment\Exception\AuthenticationException;
use Drupal\commerce_payment\Exception\DeclineException;
use Drupal\commerce_price\Price;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\DependencyInjection\ContainerInterface;
use GuzzleHttp\ClientInterface;
use Drupal\commerce_payme\HandlePaymentReturn;

/**
 * Provides the Off-site Redirect payment gateway.
 *
 * @CommercePaymentGateway(
 *   id = "payme",
 *   label = "Commerce BNCR (Redirect to BNCR)",
 *   display_label = "BNCR",
 *   forms = {
 *     "offsite-payment" = "Drupal\commerce_payme\PluginForm\PaymeForm"
 *   },
 *   payment_method_types = {"credit_card"},
 *   credit_card_types = {
 *     "amex", "dinersclub", "discover", "jcb", "mastercard", "visa"
 *   },
 * )
 */
class Payme extends OffsitePaymentGatewayBase {

  /**
   * HTTP Client.
   *
   * @var \GuzzleHttp\Client
   */
  protected $client;

  /**
   * Handle payment return.
   *
   * @var \Drupal\commerce_payme\HandlePaymentReturn
   */
  protected $handlePaymentReturn;

  /**
   * Url.
   *
   * @var string
   */
  protected $url;

  /**
   * Constructs a new Offsite object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\commerce_payment\PaymentTypeManager $payment_type_manager
   *   The payment type manager.
   * @param \Drupal\commerce_payment\PaymentMethodTypeManager $payment_method_type_manager
   *   The payment method type manager.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The time.
   * @param \GuzzleHttp\ClientInterface $client
   *   The http client.
   * @param \Drupal\commerce_payme\HandlePaymentReturn $handle_payment_return
   *   The handle payment return service.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_type_manager, PaymentTypeManager $payment_type_manager, PaymentMethodTypeManager $payment_method_type_manager, TimeInterface $time, ClientInterface $client, HandlePaymentReturn $handle_payment_return) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $entity_type_manager, $payment_type_manager, $payment_method_type_manager, $time);
    $this->client = $client;
    $this->handlePaymentReturn = $handle_payment_return;
    $this->url = 'https://integracion.alignetsac.com';
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('plugin.manager.commerce_payment_type'),
      $container->get('plugin.manager.commerce_payment_method_type'),
      $container->get('datetime.time'),
      $container->get('http_client'),
      $container->get('commerce_payme.handle_return')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'key' => '',
      'acquirer_id' => '',
      'commerce_id' => '',
      'currency_code' => '',
      'language' => '',
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $form['key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Key'),
      '#default_value' => $this->configuration['key'],
      '#required' => TRUE,
    ];

    $form['acquirer_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Acquirer ID'),
      '#default_value' => $this->configuration['acquirer_id'],
      '#required' => TRUE,
    ];

    $form['commerce_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Commerce ID'),
      '#default_value' => $this->configuration['commerce_id'],
      '#required' => TRUE,
    ];

    $form['currency_code'] = [
      '#type' => 'select',
      '#title' => $this->t('Currency'),
      '#default_value' => $this->configuration['currency_code'],
      '#options' => [
        '188' => $this->t('Costa Rican Colon'),
        '840' => $this->t('US Dollar'),
      ],
      '#required' => TRUE,
    ];

    $form['language'] = [
      '#type' => 'select',
      '#title' => $this->t('Language'),
      '#default_value' => $this->configuration['language'],
      '#options' => [
        'SP' => $this->t('Spanish'),
        'EN' => $this->t('Englush'),
      ],
      '#required' => TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function buildPaymentOperations(PaymentInterface $payment) {
    $operations = [];
    return $operations;
  }

  /**
   * Returns Key.
   */
  public function getKey() {
    return $this->configuration['key'] ?: '';
  }

  /**
   * Returns Acquirer ID.
   */
  public function getAcquirerId() {
    return $this->configuration['acquirer_id'] ?: '';
  }

  /**
   * Returns Commerce ID.
   */
  public function getCommerceId() {
    return $this->configuration['commerce_id'] ?: '';
  }

  /**
   * Returns Currency Code.
   */
  public function getCurrencyCode() {
    return $this->configuration['currency_code'] ?: '';
  }

  /**
   * Returns Language.
   */
  public function getLanguage() {
    return $this->configuration['language'] ?: '';
  }

  /**
   * Returns url.
   */
  public function getUrl() {
    return $this->url;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);

    if (!$form_state->getErrors()) {
      $values = $form_state->getValue($form['#parents']);
      $this->configuration['key'] = $values['key'];
      $this->configuration['acquirer_id'] = $values['acquirer_id'];
      $this->configuration['commerce_id'] = $values['commerce_id'];
      $this->configuration['currency_code'] = $values['currency_code'];
      $this->configuration['language'] = $values['language'];
    }
  }

  /**
   * {@inheritdoc}
   */
  public function onReturn(OrderInterface $order, Request $request) {
    try {
      $payment_storage = $this->entityTypeManager->getStorage('commerce_payment');
      $query = $payment_storage->getQuery()
        ->condition('order_id', $order->id());
      $payment_ids = $query->execute();
      if (!count($payment_ids)) {
        $code = $request->request->get('code');
        if ($code === 1) {
          throw new DeclineException($this->t('Denied transaction'));
        }
        elseif ($code === 2) {
          throw new AuthenticationException($this->t('Rejected transaction.'));
        }
        else {
          throw new InvalidResponseException($this->t('Error in payment processing'));
        }
      }
    }
    catch (PaymentGatewayException $e) {
      // Log and throw again.
      watchdog_exception('commerce_payme', $e);
      throw $e;
    }
  }

  /**
   * Returns hash from provided values.
   */
  public function getHash($values) {
    $string = implode('', $values);
    return openssl_digest($string, 'sha512');
  }

  /**
   * {@inheritdoc}
   */
  public function deletePaymentMethod(PaymentMethodInterface $payment_method) {
    $payment_method->delete();
  }

}
